package xyz.etztech.qol.commands;

import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import org.bukkit.ChatColor;
import org.bukkit.command.Command;
import org.bukkit.command.TabExecutor;
import org.bukkit.command.CommandSender;
import xyz.etztech.qol.QoL;
import xyz.etztech.qol.EtzTechUtil;
import xyz.etztech.qol.Lang;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class WikiCommand implements TabExecutor {
    QoL plugin;

    public WikiCommand(QoL paramQoL) {
        this.plugin = paramQoL;
        plugin.getCommand("wiki").setExecutor(this);
    }

    @Override
    public boolean onCommand(CommandSender commandSender, Command command, String label, String[] args) {
        if (!commandSender.hasPermission("qol.wiki")) {
            EtzTechUtil.sms(commandSender, Lang.NO_PERMISSION.getDef());
            return true;
        }
        if (args.length < 1) {
            EtzTechUtil.sms(commandSender, ChatColor.RED + "Please give a search argument.");
            return true;
        }

        String query = String.join(" ", args);
        List<List<String>> result = getWikiResults(query, 1);

        if (result.isEmpty())
        {
            EtzTechUtil.sms(commandSender, ChatColor.RED + String.format("Nothing was found on the wiki using: %s", query));
            return true;
        }

        String message = ChatColor.GREEN + result.get(0).get(0) + ": " + ChatColor.WHITE + result.get(0).get(1);
        EtzTechUtil.sms(commandSender, message);

        return true;
    }

    @Override
    public List<String> onTabComplete(CommandSender commandSender, Command command, String label, String[] args) {
        String query = String.join(" ", args);

        if (query.isEmpty()) return null;

        List<List<String>> results = getWikiResults(query, 10);

        if (results.isEmpty()) return null;

        List<String> completions = new ArrayList<>();
        results.forEach((result) -> completions.add(result.get(0)));
        return completions;
    }

    private List<List<String>> getWikiResults(String query, Integer numOfResults) {
        try {
            InputStream response = new URL(String.format(plugin.getConfig().getString("wiki-base-url", "https://minecraft.wiki:") + "/api.php?action=opensearch&format=json&formatversion=2&search=%s&namespace=0&limit=%d", query, numOfResults)).openStream();
            BufferedReader reader = new BufferedReader(new InputStreamReader(response));
            JsonArray jsonArray = new JsonParser().parse(reader.readLine()).getAsJsonArray();

            JsonArray names = jsonArray.get(1).getAsJsonArray();
            JsonArray links = jsonArray.get(3).getAsJsonArray();

            // Check if we got any result
            if (names.size() < 1)
                return Collections.emptyList();

            List<List<String>> results = new ArrayList<>();
            for (int i = 0; i < names.size(); i++)
            {
                List<String> e = new ArrayList<>();
                e.add(names.get(i).getAsString());
                e.add(links.get(i).getAsString());
                results.add(e);
            }

            return results;
        } catch(Exception e) {
            return Collections.emptyList();
        }
    }
}
